@@ -19,6 +19,8 @@ package org.apache.maven.tools.plugin.extractor.annotations;
  * under the License.
  */
 
+import java.util.List;
+
 import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
 import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoFailureException;
@@ -27,7 +29,6 @@ import org.apache.maven.plugins.annotations.Execute;
 import org.apache.maven.plugins.annotations.LifecyclePhase;
 import org.apache.maven.plugins.annotations.Mojo;
 import org.apache.maven.plugins.annotations.Parameter;
-import org.codehaus.plexus.compiler.manager.CompilerManager;
 
 /**
  * @author Olivier Lamy
@@ -44,6 +45,14 @@ public class FooMojo
     @Parameter( property = "thebar", required = true, defaultValue = "coolbar" )
     protected String bar;
 
+    /**
+     * Setter method for Parameter field
+     */
+    public void setBar( String bar )
+    {
+        this.bar = bar;
+    }
+
     /**
      * beer for non french folks
      * @deprecated wine is better
@@ -53,10 +62,48 @@ public class FooMojo
     protected String beer;
 
     /**
-     * Plexus compiler manager.
+     * Field for setter method
      */
-    @Component
-    protected CompilerManager compilerManager;
+    private String paramFromSetter;
+
+    /**
+     * setter as parameter.
+     */
+    @Parameter( property = "props.paramFromSetter" )
+    public void setParamFromSetter(String value)
+    {
+        this.paramFromSetter = paramFromSetter;
+    }
+
+    /**
+     * add method as parameter.
+     */
+    @Parameter( property = "props.paramFromAdd" )
+    public void addParamFromAdd(String value)
+    {
+        // empty
+    }
+
+    /**
+     * deprecated setter as parameter.
+     *
+     * @deprecated reason of deprecation
+     */
+    @Deprecated
+    @Parameter( property = "props.paramFromSetterDeprecated" )
+    public void setParamFromSetterDeprecated( List<String> value)
+    {
+        // empty
+    }
+
+    /**
+     * Static methods should be excluded.
+     */
+    @Parameter
+    public static void setStaticMethod( String value )
+    {
+        // empty
+    }
 
     /**
      *
@@ -70,4 +117,10 @@ public class FooMojo
     {
         // nothing
     }
+
+    @Deprecated
+    public void deprecatedMethod(String value)
+    {
+
+    }
 }
